#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <linux/fcntl.h>
#include <linux/unistd.h>
#include <sys/ioctl.h>
#include <string.h>
#include "ioctl_man.h"

void test_IOCTL_WR_REG_OFFSET(){
	int fd = 0;
	SADDR_DATA add1;
	
	char *filename="/dev/PCIe_1553UDx_0";
	fd=open(filename, O_RDWR);
	printf("Device opened 0 \n");

	add1.daddr = 0x4184;
	add1.data = 0x1000000;
	
	if (ioctl(fd, IOCTL_WR_REG_OFFSET, &add1)){
		printf("Error writing to 0x%x reg.\n", add1.daddr);
	}
	else{
		printf("0x%x was written to 0x%x reg.\n", add1.data, add1.daddr);
	}

	close(fd);
	printf("Device closed 0 \n");
}

void test_IOCTL_RD_REG_OFFSET(){
	int fd = 0;
	SADDR_DATA add1;
	
	char *filename="/dev/PCIe_1553UDx_0";
	fd=open(filename, O_RDWR);
	printf("Device opened 0 \n");

	add1.daddr = 0x4104;
	if (ioctl(fd, IOCTL_RD_REG_OFFSET, &add1)){
		printf("Error reading from 0x%x reg.\n", add1.daddr);
	}
	else{
		printf("0x%x was read from 0x%x reg.\n", add1.data, add1.daddr);
	}

	close(fd);
	printf("Device closed 0 \n");
}

void test_IOCTL_SWITCH_MODE(short nMode, int nCh){
	int fd = 0;

	MODE_STR mode;
	
	char *filename="/dev/PCIe_1553UDx_0";
	printf("IOCTL_SWITCH_MODE begin.\n");
	fd=open(filename, O_RDWR);
	printf("Device opened 0 \n");

	mode.mode = nMode;
	mode.nCh = nCh;

	if(ioctl(fd,IOCTL_SWITCH_MODE, &mode)){
		printf("Error setting work mode.\n");
	}
	else{
		printf("Work mode was successfully set.\n");
	}

	close(fd);
	printf("Device closed 0 \n");
}

void test_IOCTL_SET_BUS(short nBus, int nCh){
	int fd = 0;

	BUS_SEL BusSel;
	
	char *filename="/dev/PCIe_1553UDx_0";
	printf("IOCTL_SET_BUS begin.\n");
	fd=open(filename, O_RDWR);
	printf("Device opened 0 \n");

	BusSel.nBus = nBus;
	BusSel.nCh = nCh;

	if(ioctl(fd, IOCTL_SET_BUS, &BusSel)){
		printf("Error setting bus number.\n");
	}
	else{
		printf("Bus number was successfully set.\n");
	}

	close(fd);
	printf("Device closed 0 \n");
}

void test_IOCTL_WORK_ENABLE(int nCh){
	int fd = 0;

	WORK_ENABLE WorkEn;
	
	char *filename="/dev/PCIe_1553UDx_0";
	printf("IOCTL_WORK_ENABLE begin.\n");
	fd=open(filename, O_RDWR);
	printf("Device opened 0 \n");

	WorkEn.nFlag = ON;
	WorkEn.nCh = nCh;

	if(ioctl(fd, IOCTL_WORK_ENABLE, &WorkEn)){
		printf("Error work enabling.\n");
	}
	else{
		printf("work was successfully enabled.\n");
	}

	close(fd);
	printf("Device closed 0 \n");
}

void test_IOCTL_CLEAR_DMA(int nChNum){
	int fd = 0;
	CH_NUM nCh;
	
	char *filename="/dev/PCIe_1553UDx_0";
	fd=open(filename, O_RDWR);
	printf("Device opened 0 \n");

	nCh.nCh=nChNum;

	if(ioctl(fd, IOCTL_CLEAR_DMA, &nCh)){
		printf("Error clearing DMA.\n");
	}
	else{
		printf("DMA index was successfully cleared.\n");
	}

	close(fd);
	printf("Device closed 0 \n");
}


void test_IOCTL_GET_NBLOCK_RAW_DMA(int nCh){
	int fd = 0;
	GET_NBLOCK GetNBlock;
	
	char *filename="/dev/PCIe_1553UDx_0";
	printf("IOCTL_GET_NBLOCK_RAW_DMA begin.\n");
	fd=open(filename, O_RDWR);
	printf("Device opened 0 \n");

	GetNBlock.nCh=nCh;

	if(ioctl(fd, IOCTL_GET_NBLOCK_RAW_DMA, &GetNBlock)){
		printf("Error getting NBLOCK RAW DMA.\n");
	}
	else{
		printf("There is %d of new data block(s).\n", GetNBlock.nBlock);
	}

	close(fd);
	printf("Device closed 0 \n");
}

void test_IOCTL_RD_RAW_DMA(short nCh){
	struct {
		unsigned int number_block;
		int nCh;
		int b[32];
	} d;

	int fd = 0, i=0;
	
	char *filename="/dev/PCIe_1553UDx_0";
	fd=open(filename, O_RDWR);
	printf("Device opened 0 \n");
	
	d.number_block=1;
	d.nCh=nCh;

	if(ioctl(fd, IOCTL_RD_RAW_DMA, &d)){
		printf("Error reading DMA.\n");
	}
	else{
		printf("Was read %d data blocks from DMA\n", d.number_block);
	}

	printf("Data:\n");
	for (i; i<32; i++){
		printf("0x%x\n", d.b[i]);
	}

	close(fd);
	printf("Device closed 0 \n");
}

void test_IOCTL_ENABLE_DMA(){
	int fd = 0;
	
	char *filename="/dev/PCIe_1553UDx_0";
	printf("IOCTL_ENABLE_DMA begin.\n");
	fd=open(filename, O_RDWR);
	printf("Device opened 0 \n");

	if(ioctl(fd, IOCTL_ENABLE_DMA, 0)){
		printf("IOCTL_ENABLE_DMA error.\n");
	}
	else{
		printf("DMA has been enabled.\n");
	}

	close(fd);
	printf("Device closed 0 \n");
}

void test_IOCTL_DISABLE_DMA(){
	int fd = 0;
	
	char *filename="/dev/PCIe_1553UDx_0";
	printf("IOCTL_DISABLE_DMA begin.\n");
	fd=open(filename, O_RDWR);
	printf("Device opened 0 \n");

	if(ioctl(fd, IOCTL_DISABLE_DMA, 0)){
		printf("IOCTL_DISABLE_DMA error.\n");
	}
	else{
		printf("DMA has been disabled.\n");
	}

	close(fd);
	printf("Device closed 0 \n");
}

void test_IOCTL_ENABLE_SUBADDR(short nCh){
	int fd = 0;
	SADDR_DATA_PA sadpa;
	
	char *filename="/dev/PCIe_1553UDx_0";
	printf("IOCTL_ENABLE_PADDR begin.\n");
	fd=open(filename, O_RDWR);
	printf("Device opened 0 \n");

	sadpa.nRT = RECEIVE;
	sadpa.paddr = 1;
	sadpa.nCh = nCh;
	
	if(ioctl(fd, IOCTL_ENABLE_SUBADDR, &sadpa)){
		printf("IOCTL_ENABLE_SUBADDR error.\n");
	}
	else{
		printf("SubAddress number 1 was enable for receive.\n");
	}

	close(fd);
	printf("Device closed 0 \n");
}

void test_IOCTL_DISABLE_SUBADDR(short nCh){
	int fd = 0;
	SADDR_DATA_PA sadpa;
	
	char *filename="/dev/PCIe_1553UDx_0";
	printf("IOCTL_DISABLE_SUBADDR begin.\n");
	fd=open(filename, O_RDWR);
	printf("Device opened 0 \n");

	sadpa.paddr = 1;
	sadpa.nCh = nCh;
	
	if(ioctl(fd, IOCTL_DISABLE_SUBADDR, &sadpa)){
		printf("IOCTL_DISABLE_SUBADDR error.\n");
	}
	else{
		printf("SubAddress number 1 was DISABLE.\n");
	}

	close(fd);
	printf("Device closed 0 \n");
}

void test_IOCTL_SET_ADDRESS(int addr, short nCh){
	int fd = 0;
	AD_STR sad;
	
	char *filename="/dev/PCIe_1553UDx_0";
	printf("IOCTL_SET_ADDRESS begin.\n");
	fd=open(filename, O_RDWR);
	printf("Device opened 0 \n");

	sad.ad = addr;
	sad.nCh=nCh;
	
	if(ioctl(fd, IOCTL_SET_ADDRESS, &sad)){
		printf("IOCTL_SET_ADDRESS error.\n");
	}
	else{
		printf("Address %d was assigned to the device.\n", sad.ad);
	}

	close(fd);
	printf("Device closed 0 \n");
}

void test_IOCTL_WR_BLOCK_BUF_PA(int nCh){
	int fd = 0, i=0;
	SBLOCK_BUF_PA sbbpa;
	
	char *filename="/dev/PCIe_1553UDx_0";
	printf("IOCTL_WR_BLOCK_BUF_PA begin.\n");
	fd=open(filename, O_RDWR);
	printf("Device opened 0 \n");

	sbbpa.num_buf = 0;
	sbbpa.paddr = 1;
	sbbpa.nCh=nCh;

	for(i=0; i<31; i++)
		sbbpa.buf[i]=0x1234;
	
	if(ioctl(fd, IOCTL_WR_BLOCK_BUF_PA, &sbbpa)){
		printf("IOCTL_WR_BLOCK_BUF_PA error.\n");
	}
	else{
		printf("sbbpa.buf was written to %d buf, PAddress %d.\n", sbbpa.num_buf, sbbpa.paddr);
	}

	close(fd);
	printf("Device closed 0 \n");
}

void test_IOCTL_RD_BLOCK_BUF_PA(int nCh){
	int fd = 0, i=0;
	SBLOCK_BUF_PA sbbpa;
	
	char *filename="/dev/PCIe_1553UDx_0";
	printf("IOCTL_RD_BLOCK_BUF_PA begin.\n");
	fd=open(filename, O_RDWR);
	printf("Device opened 0 \n");

	sbbpa.num_buf = 0;
	sbbpa.paddr = 1;
	sbbpa.nCh=nCh;
	
	
	if(ioctl(fd, IOCTL_RD_BLOCK_BUF_PA, &sbbpa)){
		printf("IOCTL_RD_BLOCK_BUF_PA error.\n");
	}
	else{
		printf("Data has been read from %d buf, PAddress %d to sbbpa.buf.\n", sbbpa.num_buf, sbbpa.paddr);
	}

	for (i; i<31; i++){
		printf("0x%x\n", sbbpa.buf[i]);
	}

	close(fd);
	printf("Device closed 0 \n");
}



void test_IOCTL_VERSION(){
	int fd = 0;
	VERSION ver;
	
	char *filename="/dev/PCIe_1553UDx_0";
	printf("IOCTL_VERSION begin.\n");
	fd=open(filename, O_RDWR);
	printf("Device opened 0 \n");
	
	if(ioctl(fd, IOCTL_VERSION, &ver)){
		printf("IOCTL_VERSION error.\n");
	}
	else{
		printf("Number of channels = %d,\nrevision = 0x%x, \nDMA size = 0x%x. \n", ver.type, ver.revision, ver.size_dma);
	}

	close(fd);
	printf("Device closed 0 \n");
}

void test_IOCTL_SET_TR_PROG_MODE(int nCh){
	int fd = 0;
	SET_TR_MODE SetTRMode;
	
	char *filename="/dev/PCIe_1553UDx_0";
	printf("IOCTL_SET_TR_PROG_MODE begin.\n");
	fd=open(filename, O_RDWR);
	printf("Device opened 0 \n");

	SetTRMode.nCh=nCh;
	SetTRMode.nNum=1;
	
	if(ioctl(fd, IOCTL_SET_TR_PROG_MODE, &SetTRMode)){
		printf("IOCTL_SET_TR_PROG_MODE error.\n");
	}
	else{
		printf("Transmit mode was successfuly set.\n");
	}

	close(fd);
	printf("Device closed 0 \n");
}

void test_IOCTL_SET_TR_BUF_READY(int nCh){
	int fd = 0;
	SET_TR_MODE_BUF SetTRModeBuf;
	
	char *filename="/dev/PCIe_1553UDx_0";
	printf("IOCTL_SET_TR_BUF_READY begin.\n");
	fd=open(filename, O_RDWR);
	printf("Device opened 0 \n");

	SetTRModeBuf.nCh=nCh;
	SetTRModeBuf.nNum=1;
	SetTRModeBuf.nBuf=BUF0;
	
	if(ioctl(fd, IOCTL_SET_TR_BUF_READY, &SetTRModeBuf)){
		printf("IOCTL_SET_TR_BUF_READY error.\n");
	}
	else{
		printf("Buffer was successfuly set fo ready state.\n");
	}

	close(fd);
	printf("Device closed 0 \n");
}

void test_IOCTL_SUBADDRESS_IRQ_TRANSMIT(int nCh){
	int fd = 0;
	SADDR_INT saddr_int;
	
	char *filename="/dev/PCIe_1553UDx_0";
	printf("IOCTL_SET_TR_BUF_READY begin.\n");
	fd=open(filename, O_RDWR);
	printf("Device opened 0 \n");

	saddr_int.nCh=nCh;
	saddr_int.nNum=1;
	saddr_int.action=DISABLE;//ENABLE;
	
	if(ioctl(fd, IOCTL_SUBADDRESS_IRQ_TRANSMIT, &saddr_int)){
		printf("IOCTL_SUBADDRESS_IRQ_TRANSMIT error.\n");
	}
	else{
		printf("Transmit Subaddress was enabled/disable for interrupts.\n");
	}

	close(fd);
	printf("Device closed 0 \n");
}

void test_IOCTL_SUBADDRESS_IRQ_RECEIVE(int nCh){
	int fd = 0;
	SADDR_INT saddr_int;
	
	char *filename="/dev/PCIe_1553UDx_0";
	printf("IOCTL_SUBADDRESS_IRQ_RECEIVE begin.\n");
	fd=open(filename, O_RDWR);
	printf("Device opened 0 \n");

	saddr_int.nCh=nCh;
	saddr_int.nNum=1;
	saddr_int.action=DISABLE;//ENABLE;
	
	if(ioctl(fd, IOCTL_SUBADDRESS_IRQ_RECEIVE, &saddr_int)){
		printf("IOCTL_SUBADDRESS_IRQ_RECEIVE error.\n");
	}
	else{
		printf("Receive Subaddress was enabled/disable for interrupts.\n");
	}

	close(fd);
	printf("Device closed 0 \n");
}

void test_IOCTL_SET_TIMER_RTBM_CONF_REG_PCI(int nCh){
	int fd = 0;
	TT tt;
	
	char *filename="/dev/PCIe_1553UDx_0";
	printf("IOCTL_SET_TIMER_RTBM_CONF_REG_PCI begin.\n");
	fd=open(filename, O_RDWR);
	printf("Device opened 0 \n");

	tt.nCh=nCh;
	tt.timeout_rcv=17;
	tt.timeout_tr=6;
	tt.timer=2;
	
	if(ioctl(fd, IOCTL_SET_TIMER_RTBM_CONF_REG_PCI, &tt)){
		printf("IOCTL_SET_TIMER_RTBM_CONF_REG_PCI error.\n");
	}
	else{
		printf("Timeout values has been set.\n");
	}

	close(fd);
	printf("Device closed 0 \n");
}

void test_IOCTL_SET_TIMER_BC_CONF_REG_PCI (int nCh){
	int fd = 0;
	TT tt;
	
	char *filename="/dev/PCIe_1553UDx_0";
	printf("IOCTL_SET_TIMER_BC_CONF_REG_PCI  begin.\n");
	fd=open(filename, O_RDWR);
	printf("Device opened 0 \n");

	tt.nCh=nCh;
	tt.timeout_rcv=17;
	tt.timeout_tr=6;
	tt.timer=2;
	
	if(ioctl(fd, IOCTL_SET_TIMER_BC_CONF_REG_PCI , &tt)){
		printf("IOCTL_SET_TIMER_BC_CONF_REG_PCI  error.\n");
	}
	else{
		printf("Timeout values has been set.\n");
	}

	close(fd);
	printf("Device closed 0 \n");
}

int main(int argc, char *argv[]){

	int nCh = 2;
	
	//test_IOCTL_WR_REG_OFFSET();
	//test_IOCTL_RD_REG_OFFSET();
	//test_IOCTL_SWITCH_MODE(RT_MODE, nCh);
	//test_IOCTL_CLEAR_DMA(nCh);
	//test_IOCTL_GET_NBLOCK_RAW_DMA(nCh);
	//test_IOCTL_RD_RAW_DMA();
	//test_IOCTL_ENABLE_DMA();
	//test_IOCTL_DISABLE_DMA();
	//test_IOCTL_SET_BUS(BOTH, nCh);
	//test_IOCTL_ENABLE_CHANNEL();
	//test_IOCTL_DISABLE_CHANNEL();
	//test_IOCTL_SET_ADDRESS(3);
	//test_IOCTL_SET_TIMER_TIMEOUTS();
	//test_IOCTL_ENABLE_SUBADDR(nCh);
	//test_IOCTL_DISABLE_SUBADDR(nCh);
	//test_IOCTL_WR_REG_PA_RT();
	//test_IOCTL_RD_REG_PA_RT();
	//test_IOCTL_WR_REG_PA_DIS(BM_ADDR_MODE);		//Work mode should be set to BM_ADDR_MODE or BM_MODE
	//test_IOCTL_RD_REG_PA_DIS(BM_ADDR_MODE);		//Work mode should be set to BM_ADDR_MODE or BM_MODE
	//test_IOCTL_WR_BLOCK_BUF_PA(BM_ADDR_MODE);		//Work mode should be set to BM_ADDR_MODE or RT_MODE
	//test_IOCTL_RD_BLOCK_BUF_PA(BM_ADDR_MODE);		//Work mode should be set to BM_ADDR_MODE or RT_MODE
	//test_IOCTL_WRITE_BC_CONTR_REG(BC_MODE);		//Work mode should be set to BC_MODE
	//test_IOCTL_READ_BC_CONTR_REG(BC_MODE);		//Work mode should be set to BC_MODE
	
	//test_IOCTL_VERSION();
	//test_IOCTL_WR_BLOCK_BUF_PA(nCh);
	//test_IOCTL_RD_BLOCK_BUF_PA(nCh);

	
	//Module initialization
	/*test_IOCTL_SWITCH_MODE(RT_MODE, nCh);
	test_IOCTL_SET_ADDRESS(2, nCh);
	test_IOCTL_SET_BUS(BOTH, nCh);
	test_IOCTL_ENABLE_SUBADDR(nCh);
	test_IOCTL_WORK_ENABLE(nCh);
	test_IOCTL_ENABLE_DMA();*/

	//Read new data
	//test_IOCTL_GET_NBLOCK_RAW_DMA(nCh);
	//test_IOCTL_RD_RAW_DMA(nCh);

	//Send Data
	/*test_IOCTL_WR_BLOCK_BUF_PA(nCh);
	test_IOCTL_RD_BLOCK_BUF_PA(nCh);
	test_IOCTL_SET_TR_PROG_MODE(nCh);	
	test_IOCTL_SET_TR_BUF_READY(nCh);*/

	//test_IOCTL_SUBADDRESS_IRQ_TRANSMIT(nCh);
	//test_IOCTL_SUBADDRESS_IRQ_RECEIVE(nCh);


	





	

}
